@@ -50,6 +50,7 @@ static void help(void);
 static void setcom(int);
 int decide_tag_by_context(const char *, const char *, int);
 int main(int, char **);
+int completion_tags(const char *, const char *, const char *, int);
 void completion(const char *, const char *, const char *, int);
 void completion_idutils(const char *, const char *, const char *);
 void completion_path(const char *, const char *);
@@ -100,6 +101,7 @@ const char *cwd, *root, *dbpath;
 char *context_file;
 char *context_lineno;
 char *file_list;
+char *encode_chars;
 
 static void
 usage(void)
@@ -407,6 +409,7 @@ main(int argc, char **argv)
 			break;
 		case 'v':
 			vflag++;
+			setverbose();
 			break;
 		case 'V':
 			Vflag++;
@@ -415,11 +418,7 @@ main(int argc, char **argv)
 			xflag++;
 			break;
 		case ENCODE_PATH:
-			if (strlen(optarg) > 255)
-				die("too many encode chars.");
-			if (strchr(optarg, '/') || strchr(optarg, '.'))
-				die("cannot encode '/' and '.' in the path.");
-			set_encode_chars((unsigned char *)optarg);
+			encode_chars = optarg;
 			break;
 		case FROM_HERE:
 			{
@@ -454,6 +453,8 @@ main(int argc, char **argv)
 				format = FORMAT_CTAGS_XID;
 			else if (!strcmp(optarg, "ctags"))
 				format = FORMAT_CTAGS;
+			else if (!strcmp(optarg, "ctags-mod"))
+				format = FORMAT_CTAGS_MOD;
 			else if (!strcmp(optarg, "path"))
 				format = FORMAT_PATH;
 			else if (!strcmp(optarg, "grep"))
@@ -468,6 +469,38 @@ main(int argc, char **argv)
 			break;
 		}
 	}
+	/*
+	 * decide format.
+	 * The --result option is given to priority more than the -t and -x option.
+	 */
+	if (format == 0) {
+		if (tflag) { 			/* ctags format */
+			format = FORMAT_CTAGS;
+		} else if (xflag) {		/* print details */
+			format = FORMAT_CTAGS_X;
+		} else {			/* print just a file name */
+			format = FORMAT_PATH;
+		}
+	}
+	/*
+	 * GTAGSBLANKENCODE will be used in less(1).
+	 */
+	switch (format) {
+	case FORMAT_CTAGS_X:
+	case FORMAT_CTAGS_XID:
+		if (encode_chars == NULL && getenv("GTAGSBLANKENCODE"))
+			encode_chars = " \t";
+		break;
+	}
+	if (encode_chars) {
+		if (strlen(encode_chars) > 255)
+			die("too many encode chars.");
+		if (strchr(encode_chars, '/') || strchr(encode_chars, '.'))
+			warning("cannot encode '/' and '.' in the path. Ignored.");
+		set_encode_chars((unsigned char *)encode_chars);
+	}
+	if (getenv("GTAGSTHROUGH"))
+		Tflag++;
 	if (qflag)
 		vflag = 0;
 	if (show_help)
@@ -585,6 +618,11 @@ main(int argc, char **argv)
 	 * decide tag type.
 	 */
 	if (context_file) {
+		static char buf[MAXPATHLEN];
+
+		if (realpath(context_file, buf) == NULL)
+			die("cannot get real path name.");
+		context_file = buf;
 		if (isregex(av))
 			die_with_code(2, "regular expression is not allowed with the --from-here option.");
 		db = decide_tag_by_context(av, context_file, atoi(context_lineno));
@@ -631,19 +669,6 @@ main(int argc, char **argv)
 		fprintf(stderr, "localprefix=%s\n", localprefix);
 #endif
 	}
-	/*
-	 * decide format.
-	 * The --result option is given to priority more than the -t and -x option.
-	 */
-	if (format == 0) {
-		if (tflag) { 			/* ctags format */
-			format = FORMAT_CTAGS;
-		} else if (xflag) {		/* print details */
-			format = FORMAT_CTAGS_X;
-		} else {			/* print just a file name */
-			format = FORMAT_PATH;
-		}
-	}
 	/*
 	 * decide path conversion type.
 	 */
@@ -690,22 +715,22 @@ main(int argc, char **argv)
 	return 0;
 }
 /*
- * completion: print completion list of specified prefix
+ * completion_tags: print completion list of specified prefix
  *
  *	i)	dbpath	dbpath directory
  *	i)	root	root directory
  *	i)	prefix	prefix of primary key
  *	i)	db	GTAGS,GRTAGS,GSYMS
+ *	r)		number of words
  */
-void
-completion(const char *dbpath, const char *root, const char *prefix, int db)
+int
+completion_tags(const char *dbpath, const char *root, const char *prefix, int db)
 {
 	int flags = GTOP_KEY;
 	GTOP *gtop = gtags_open(dbpath, root, db, GTAGS_READ, 0);
 	GTP *gtp;
+	int count = 0;
 
-	if (prefix && *prefix == 0)	/* In the case global -c '' */
-		prefix = NULL;
 	if (prefix && isalpha(*prefix) && iflag) {
 		/*
 		 * If the -i option is specified, we use both of regular
@@ -745,6 +770,7 @@ completion(const char *dbpath, const char *root, const char *prefix, int db)
 				if (regexec(&preg, gtp->tag, 0, 0, 0) == 0) {
 					fputs(gtp->tag, stdout);
 					fputc('\n', stdout);
+					count++;
 				}
 			}
 		}
@@ -756,9 +782,60 @@ completion(const char *dbpath, const char *root, const char *prefix, int db)
 		for (gtp = gtags_first(gtop, prefix, flags); gtp; gtp = gtags_next(gtop)) {
 			fputs(gtp->tag, stdout);
 			fputc('\n', stdout);
+			count++;
 		}
 	}
 	gtags_close(gtop);
+	return count;
+}
+/*
+ * completion: print completion list of specified prefix
+ *
+ *	i)	dbpath	dbpath directory
+ *	i)	root	root directory
+ *	i)	prefix	prefix of primary key
+ *	i)	db	GTAGS,GRTAGS,GSYMS
+ */
+void
+completion(const char *dbpath, const char *root, const char *prefix, int db)
+{
+	int count, total = 0;
+	char libdbpath[MAXPATHLEN];
+
+	if (prefix && *prefix == 0)	/* In the case global -c '' */
+		prefix = NULL;
+	count = completion_tags(dbpath, root, prefix, db);
+	/*
+	 * search in library path.
+	 */
+	if (db == GTAGS && getenv("GTAGSLIBPATH") && (count == 0 || Tflag) && !lflag) {
+		STRBUF *sb = strbuf_open(0);
+		char *libdir, *nextp = NULL;
+
+		strbuf_puts(sb, getenv("GTAGSLIBPATH"));
+		/*
+		* search for each tree in the library path.
+		*/
+		for (libdir = strbuf_value(sb); libdir; libdir = nextp) {
+			if ((nextp = locatestring(libdir, PATHSEP, MATCH_FIRST)) != NULL)
+				*nextp++ = 0;
+			if (!gtagsexist(libdir, libdbpath, sizeof(libdbpath), 0))
+				continue;
+			if (!strcmp(dbpath, libdbpath))
+				continue;
+			if (!test("f", makepath(libdbpath, dbname(db), NULL)))
+				continue;
+			/*
+			 * search again
+			 */
+			count = completion_tags(libdbpath, libdir, prefix, db);
+			total += count;
+			if (count > 0 && !Tflag)
+				break;
+		}
+		strbuf_close(sb);
+	}
+	/* return total; */
 }
 /*
  * completion_idutils: print completion list of specified prefix
@@ -823,12 +900,14 @@ completion_path(const char *dbpath, const char *prefix)
 {
 	GFIND *gp;
 	const char *localprefix = "./";
-	FILE *op = popen("sort | uniq", "w");
+	DBOP *dbop = dbop_open(NULL, 1, 0600, DBOP_RAW);
 	const char *path;
 	int prefix_length;
 	int target = GPATH_SOURCE;
 	int flags = (match_part == MATCH_PART_LAST) ? MATCH_LAST : MATCH_FIRST;
 
+	if (dbop == NULL)
+		die("cannot open temporary file.");
 	if (prefix && *prefix == 0)	/* In the case global -c '' */
 		prefix = NULL;
 	prefix_length = (prefix == NULL) ? 0 : strlen(prefix);
@@ -840,28 +919,29 @@ completion_path(const char *dbpath, const char *prefix)
 		flags |= IGNORE_CASE;
 	gp = gfind_open(dbpath, localprefix, target);
 	while ((path = gfind_read(gp)) != NULL) {
-		path += 2;				/* skip './'*/
+		path++;					/* skip '.'*/
 		if (prefix == NULL) {
-			fputs(path, op);
-			fputc('\n', op);
+			dbop_put(dbop, path + 1, "");
 		} else if (match_part == MATCH_PART_ALL) {
 			const char *p = path;
 
 			while ((p = locatestring(p, prefix, flags)) != NULL) {
-				fputs(p, op);
-				fputc('\n', op);
+				dbop_put(dbop, p, "");
 				p += prefix_length;
 			}
 		} else {
 			const char *p = locatestring(path, prefix, flags);
 			if (p != NULL) {
-				fputs(p, op);
-				fputc('\n', op);
+				dbop_put(dbop, p, "");
 			}
 		}
 	}
 	gfind_close(gp);
-	pclose(op);
+	for (path = dbop_first(dbop, NULL, NULL, DBOP_KEY); path != NULL; path = dbop_next(dbop)) {
+		fputs(path, stdout);
+		fputc('\n', stdout);
+	}
+	dbop_close(dbop);
 }
 /*
  * Output filter
@@ -1268,12 +1348,22 @@ void
 parsefile(char *const *argv, const char *cwd, const char *root, const char *dbpath, int db)
 {
 	int count = 0;
+	int flags = 0;
 	STRBUF *sb = strbuf_open(0);
 	char *langmap;
 	const char *plugin_parser, *av;
 	char path[MAXPATHLEN];
 	struct parsefile_data data;
 
+	flags = 0;
+	if (vflag)
+		flags |= PARSER_VERBOSE;
+	if (debug)
+		flags |= PARSER_DEBUG;
+	/*
+	if (wflag)
+		flags |= PARSER_WARNING;
+	*/
 	if (db == GRTAGS + GSYMS)
 		data.target = TARGET_REF|TARGET_SYM;
 	else
@@ -1348,7 +1438,7 @@ parsefile(char *const *argv, const char *cwd, const char *root, const char *dbpa
 		if (lflag && !locatestring(path, localprefix, MATCH_AT_FIRST))
 			continue;
 		data.count = 0;
-		parse_file(path, 0, put_syms, &data);
+		parse_file(path, flags, put_syms, &data);
 		count += data.count;
 	}
 	args_close();
