@@ -77,9 +77,11 @@
 #define LINEFLAG "+%s"	/* default: used by vi and emacs */
 #define TMPDIR	"/tmp"
 
-static char const rcsid[] = "$Id: gtags-cscope.c,v 1.21 2011/08/29 03:46:44 shigio Exp $";
+static char const rcsid[] = "$Id: gtags-cscope.c,v 1.23 2011/10/07 01:07:29 shigio Exp $";
 
 char	*editor, *shell, *lineflag;	/* environment variables */
+char	*global_command;	/* "global" by default */
+char	*gtags_command;		/* "gtags" by default */
 char	*home;			/* Home directory */
 BOOL	lineflagafterfile;
 char	*argv0;			/* command name */
@@ -301,6 +303,8 @@ gtags-cscope: pattern too long, cannot be > %d characters\n", PATLEN);
     editor = mygetenv("VIEWER", editor); /* use viewer if set */
     editor = mygetenv("CSCOPE_EDITOR", editor);	/* has last word */
     home = mygetenv("HOME", HOME);
+    global_command = mygetenv("GTAGSGLOBAL", "global");
+    gtags_command = mygetenv("GTAGSGTAGS", "gtags");
 #if defined(_WIN32) || defined(__DJGPP__)
     shell = mygetenv("COMSPEC", SHELL);
     shell = mygetenv("SHELL", shell);
@@ -388,7 +392,9 @@ cscope: Could not create private temp dir %s\n",
 
     /* if the cross-reference is to be considered up-to-date */
     if (isuptodate == YES) {
-	if (execute("global", "global", "-p", NULL) != 0) {
+	char com[80];
+	snprintf(com, sizeof(com), "%s -p >/dev/null", global_command);
+	if (system(com) != 0) {
 	    postfatal("gtags-cscope: GTAGS not found. Please invoke again without -d option.\n");
             /* NOTREACHED */
 	}
