@@ -238,6 +238,7 @@ main(int argc, char **argv)
 			break;
 		case 'v':
 			vflag++;
+			setverbose();
 			break;
 		default:
 			usage();
@@ -351,6 +352,9 @@ main(int argc, char **argv)
 		else if (!test("r", file_list))
 			die("'%s' is not readable.", file_list);
 	}
+	if (!getcwd(cwd, MAXPATHLEN))
+		die("cannot get current directory.");
+	canonpath(cwd);
 	/*
 	 * Regularize the path name for single updating (--single-update).
 	 */
@@ -360,17 +364,22 @@ main(int argc, char **argv)
 		
 		if (!test("f", p))
 			die("'%s' not found.", p);
-		if (isabspath(p))
-			die("--single-update requires relative path name.");
-		if (!(p[0] == '.' && p[1] == '/')) {
-			snprintf(regular_path_name, MAXPATHLEN, "./%s", p);
-			p = regular_path_name;
+		if (isabspath(p)) {
+			char *q = locatestring(p, cwd, MATCH_AT_FIRST);
+
+			if (q && *q == '/')
+				snprintf(regular_path_name, MAXPATHLEN, "./%s", q + 1);
+			else
+				die("path '%s' is out of the project.", p);
+
+		} else {
+			if (p[0] == '.' && p[1] == '/')
+				snprintf(regular_path_name, MAXPATHLEN, "%s", p);
+			else
+				snprintf(regular_path_name, MAXPATHLEN, "./%s", p);
 		}
-		single_update = p;
+		single_update = regular_path_name;
 	}
-	if (!getcwd(cwd, MAXPATHLEN))
-		die("cannot get current directory.");
-	canonpath(cwd);
 	/*
 	 * Decide directory (dbpath) in which gtags make tag files.
 	 *
@@ -798,8 +807,6 @@ updatetags(const char *dbpath, const char *root, IDSET *deleteset, STRBUF *addli
 			die("GPATH is corrupted.('%s' not found)", path);
 		if (vflag)
 			fprintf(stderr, " [%d/%d] extracting tags of %s\n", ++seqno, total, path + 2);
-		if (debug)
-			fprintf(stderr, "[%s]\n", path + 2);
 		parse_file(path, flags, put_syms, &data);
 		gtags_flush(data.gtop[GTAGS], data.fid);
 		if (data.gtop[GRTAGS] != NULL)
@@ -864,8 +871,6 @@ createtags(const char *dbpath, const char *root)
 		seqno++;
 		if (vflag)
 			fprintf(stderr, " [%d] extracting tags of %s\n", seqno, path + 2);
-		if (debug)
-			fprintf(stderr, "[%s]\n", path + 2);
 		parse_file(path, flags, put_syms, &data);
 		gtags_flush(data.gtop[GTAGS], data.fid);
 		gtags_flush(data.gtop[GRTAGS], data.fid);
