@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004 Tama Communications Corporation
+ * Copyright (c) 2004, 2010 Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -101,3 +101,21 @@ path2fid(const char *path)
 	}
 	return p;
 }
+/*
+ * path2fid_readonly: convert the path name into the file id.
+ *
+ *	i)	path	path name
+ *	r)		id
+ */
+const char *
+path2fid_readonly(const char *path)
+{
+	if (strlen(path) > MAXPATHLEN)
+		die("path name too long. '%s'", path);
+	/*
+	 * accept both aaa and ./aaa.
+	 */
+	if (*path == '.' && *(path + 1) == '/')
+		path += 2;
+	return assoc_get(assoc, path);
+}
