@@ -57,13 +57,15 @@ set_encode_chars(const unsigned char *chars)
 	/* clean the table */
 	memset(encode, 0, sizeof(encode));
 	/* set bits */
-	encoding = 0;
 	for (i = 0; chars[i]; i++) {
 		encode[(unsigned char)chars[i]] = 1;
-		encoding = 1;
 	}
+	/* You cannot encode '.' and '/'. */
+	encode['.'] = 0;
+	encode['/'] = 0;
 	/* '%' is always encoded when encode is enable. */
 	encode['%'] = 1;
+	encoding = 1;
 }
 /*
  * set_print0: change newline to '\0'.
@@ -74,7 +76,7 @@ set_print0()
 	newline = '\0';
 }
 #define outofrange(c)	(c < '0' || c > 'f')
-#define h2int(c) (c >= 'a' ? c - 'a' : c - '0')
+#define h2int(c) (c >= 'a' ? c - 'a' + 10 : c - '0')
 /*
  * decode_path: decode encoded path name.
  *
@@ -314,6 +316,13 @@ convert_put(CONVERT *cv, const char *ctags_x)
 		fputc(' ', cv->op);
 		fputs(rest, cv->op);
 		break;
+	case FORMAT_CTAGS_MOD:
+		fputs(convert_pathname(cv, path), cv->op);
+		fputc('\t', cv->op);
+		fputs(lineno, cv->op);
+		fputc('\t', cv->op);
+		fputs(rest, cv->op);
+		break;
 	case FORMAT_GREP:
 		fputs(convert_pathname(cv, path), cv->op);
 		fputc(':', cv->op);
@@ -324,10 +333,7 @@ convert_put(CONVERT *cv, const char *ctags_x)
 	case FORMAT_CSCOPE:
 		fputs(convert_pathname(cv, path), cv->op);
 		fputc(' ', cv->op);
-		if (cv->db == GTAGS)
-			fputs(tag, cv->op);
-		else
-			fputs("<unknown>", cv->op);
+		fputs(tag, cv->op);
 		fputc(' ', cv->op);
 		fputs(lineno, cv->op);
 		fputc(' ', cv->op);
@@ -394,6 +400,13 @@ convert_put_using(CONVERT *cv, const char *tag, const char *path, int lineno, co
 		fprintf(cv->op, "%-16s %4d %-16s %s",
 			tag, lineno, convert_pathname(cv, path), rest);
 		break;
+	case FORMAT_CTAGS_MOD:
+		fputs(convert_pathname(cv, path), cv->op);
+		fputc('\t', cv->op);
+		fprintf(cv->op, "%d", lineno);
+		fputc('\t', cv->op);
+		fputs(rest, cv->op);
+		break;
 	case FORMAT_GREP:
 		fputs(convert_pathname(cv, path), cv->op);
 		fputc(':', cv->op);
@@ -404,10 +417,7 @@ convert_put_using(CONVERT *cv, const char *tag, const char *path, int lineno, co
 	case FORMAT_CSCOPE:
 		fputs(convert_pathname(cv, path), cv->op);
 		fputc(' ', cv->op);
-		if (cv->db == GTAGS)
-			fputs(tag, cv->op);
-		else
-			fputs("<unknown>", cv->op);
+		fputs(tag, cv->op);
 		fputc(' ', cv->op);
 		fprintf(cv->op, "%d", lineno);
 		fputc(' ', cv->op);
